
##MAIN FILE WITH FUNCTIONS OF CLASSIFICATION WITHIN THE LEIBNIZ HIERARCHY###
import sys
import itertools

from org.uacalc.alg import *
from org.uacalc.terms import *

from java.util import *
from java.lang import Integer

from AuxItertools import *
from LeibCongruence import *


#internal function. For each binary term (i.e., t in the free algebra with two generators)
#obtain a set delta that is reflexive in filter (i.e. delta(f,f) in filter for all f in filter) and such that 
#for each (a,b) in mp there is delta in Delta such that delta(a,b) is not in filter
#input: logic (class Logic), mp [[int,int]]
#
#output: [False, []] if there is not such set Delta
#		 [True, Delta] otherwise, with Delta [Term]
def _reflexAndmpClean(logic, mp):
    universe = logic.getUniverse()
    freeTerms = logic.getFree2().getTerms()
    filter = logic.getFilter()

    algebra = logic.getAlgebra()
    x = VariableImp("x")
    y = VariableImp("y")
    assign = HashMap()
    delta = []
    p = 0
    #print "mp:", mp
    for term in freeTerms:
		#print "analising term ", term.toString()
		#check reflexivity
		r = 1
		for a in universe:
		    assign.put(x,Integer(a))
		    assign.put(y,Integer(a))
		    v = term.intEval(algebra, assign)
		    #print "v(",a," ", a,") =", v
		    if v not in filter:	
		        #not reflexive
		        r = 0
		        break
		if r == 1:
		    #it was a reflexive term
		    #print "it was reflexive. More? "
		    #print "term ", term.toString(), " was reflexive" 
		    somefalse=0
		    notmp = []
		    for m in mp:
		        assign.put(x,Integer(m[0]))
		        assign.put(y,Integer(m[1]))
		        v = term.intEval(algebra, assign)
		        if v not in filter:
		            somefalse=1
		            #print "term ", term.toString(), " falsed MP over the pair ", m 
		            notmp.append(m)
		    if somefalse == 1:
		        delta.append(term)
		        newmp = [s for s in mp if s not in notmp]
		        mp = newmp
		        if mp == []:
		            return [True, delta]
    if p != 1:
	    return [False, []]
	



def protoalgebraic(logic):
###IN THE WAY TOWARDS PROTOALGEBRAICITY###
	#mp = list(product(filter, [x for x in universe if x not in filter]))
	#usual modus ponens
    #print logic.getUniverse()
    #print logic.getFilter()
    
    mp = [[x,y] for x in logic.getFilter() for y in logic.getUniverse() if y not in logic.getFilter()]
    #print "hola"
    return _reflexAndmpClean(logic, mp)
			
		
def equivalential(logic):
###IN THE WAY TOWARDS EQUIVALENTIALITY###
	##IMPORTANT:: IT IS ASSUMED THE MATRIX HAS BEEN REDUCED. SEE LEIBCONGRUENCE OPERATIONS OTHERWISE
	
	#mp = [[x,y] for x in logic.getUniverse() for y in logic.getUniverse() if logic.elementIndex(x) < logic.elementIndex(y)]
	mp = [[x,y] for x in logic.getUniverse() for y in logic.getUniverse() if x != y]
    
	return _reflexAndmpClean(logic, mp)

def _getKeyByValue(mapVals, value):
    for entry in mapVals.entrySet():
        if value == entry.getValue():
            return entry.getKey()
    return -1

import ast

def truthEquational(logic):
	filter = logic.getFilter()
	algebra = logic.getAlgebra()
	universe = logic.getUniverse()
	free1 = logic.getFree1()
	terms = free1.getTerms()

	eqs = [[t1,t2] for t1 in terms for t2 in terms if terms.index(t1) < terms.index(t2)]
	
	assign = HashMap()
	tau = []
	x = VariableImp("x")
	for eq in eqs:
		add = 1
		#print "checking :", eq[0].toString(), "=", eq[1].toString()
		for f in filter:
			assign.put(x,Integer(f))
			v0 = eq[0].eval(algebra, assign)
			#print "term:", eq[1].toString()
			#print "eval:", eq[1].eval(algebra, assign)
			v1 = eq[1].eval(algebra, assign)
			
			if v0 != v1:
				add = 0
				#print "failed :", eq[0].toString(), "=", eq[1].toString()
				#print "with f = ", f
				#print "eq[0].intEval(algebra, assign)", v0
				#print "eq[1].intEval(algebra, assign)", v1
				break
		if add == 1:
		  #print "adding :", eq[0].toString(), "=", eq[1].toString()
		  tau.append(eq)
	
	#generation of the filters of logic on free1
	basicFilters = []
	#print universe
	for a in universe:
		assign.put(x,Integer(a))
		nbfilter = [t for t in terms if t.eval(algebra, assign) in filter]
		if nbfilter not in basicFilters:
		  basicFilters.append(nbfilter)
	#print "basicFilters"
	filters = []
	#print "BASIC FILTERS:", basicFilters
	if len(basicFilters) <= 2:
	   for bf in basicFilters:
	       filters.append(bf)
	   inter = intersect(basicFilters)
	   if inter not in basicFilters:
	       filters.append(inter)
	elif len(basicFilters) > 2:
	    possibles = [combinations(basicFilters, n) for n in range(2, len(basicFilters)+1)]
	    #print "POSSIBLES", possibles
	    afilters = []
	    for vf in possibles:
	       #print "vf:", vf
	       l = []
	       for y in vf:
	           l.append(y)
	       print "l:", l  
	       for j in l:
	           intersection = intersect(j)
	           afilters.append(intersection)
	           #if intersection != []:
	               #print "afilters.append(intersection)", intersection
	           
	    #print "AFILTERS", afilters
	    #for bf in basicFilters:
	    #   afilters.append(bf)
	    filters = list(k for k,_ in itertools.groupby(afilters))
	    #print "FILTERS", filters
	#the rest of the filters can be generated as intersections of these ones ::see Tommaso's proof
	#print "FILTERS", filters
	

	
	
	#
	termMap = free1.getTermMap()
	filtersElements = [[_getKeyByValue(termMap, val) for val in y] for y in filters]
	#print "filtersElements", filtersElements
	#print "filters"
	#for f in filters:
	#   for el in f:
	#       for i in el:
	#           print i.toString()
	#print "free1", free1.universe()
	#print "free1Terms", terms
	
	
	for g in [f for f in filters if f != terms]:
		
		nuevag = filtersElements[filters.index(g)]
		nng = [free1.elementIndex(b) for b in nuevag]
		leibg = LeibnizCongruence(free1, nng).getPairs()
		
		
		leibReal = []
		#print type(leibg)
		#print leibg
		iterator = leibg.iterator()
		while iterator.hasNext():
		 pair = iterator.next()
		 pair = ast.literal_eval(pair.toString())
		 preal = []
		 for y in pair:
		  ele = free1.getElement(y)
		  term = termMap.get(ele)
		  preal.append(term)
		 if [preal[0],preal[1]] not in leibReal and [preal[1],preal[0]] not in leibReal:
		  leibReal.append(preal)
		
		#leibReal = list(set(leibReal))
		#leibReal = [list(x) for x in leibReal]
		#leibReal = [[termMap.get(free1.getElement(y)) for y in x] for x in leibg]
		
		#print "pairs leibniz real"
		#for cong in leibReal:
		#      print cong[0].toString(), cong[1].toString()
		vals = []
		ng = [y for y in terms if y not in g]
		#print type(ng[0])
		#print type(ng[1])
		for b in ng:
		  assign.put(x,b)
		  #print "Construction tau1"
		  for eq in tau:
		      #print "hola"
		      #print eq[0].toString()
		      #print eq[1].toString()
		      #print "type b", type(b)
		      #print "b", b.toString()
		      #print type(assign)
		      #t = eq[0]
		      #t.substitute(assign)
		      #print t.toString()
		      t0 = eq[0].substitute(assign)
		      t1 = eq[1].substitute(assign)
		      #print t0.toString()
		      #print t1.toString()
		      vals.append([t0,t1])
		      
		  #vals = list(set(vals))
		  #print "valor que falla:", b.toString()
		  #print "filter g:"
		  #for v in g:
		  #    print v
		  #print "vals"
		  #for eq in vals:
		  #    print eq[0].toString(), "=", eq[1].toString()
		  
		  #print "leibReal"
		  #for eq in leibReal:
		  #    print eq[0].toString(), "=", eq[1].toString()

		  if [z for z in vals if z in leibReal] == vals:
		      return [False, []]
    
	return [True, tau]
		
	

	