import sys
sys.path.append("Jars/uacalc.jar")
sys.path.append("Jars/LatDraw.jar")


from org.uacalc.alg.op import AbstractOperation


def operation_class_factory(fn, op_class_name, arity):
    '''Return a subclass of AbstractOperation with intValueAt() defined using the function fn.
    
    :param fn: the Python function used to define the intValueAt() method of the class.
    :param op_class_name: a string name for the class.
    :param arity: the arity of the operation (must be 1 if fn is a list or tuple)
    '''
    class OperationClass(AbstractOperation):
        # check if fn is a (callable) function
        if arity == 0:
            def intValueAt(self,args):
                return int(fn)
        if arity == 1:
            def intValueAt(self,args):
            	aux = [item[0] for item in fn]
            	i = aux.index(args[0])
            	return int(fn[i][1])
        if arity > 1:
        	def intValueAt(self,args):
        		aux = [item[0] for item in fn]
        		i = aux.index(tuple(args))
        		return int(fn[i][1])

            
    # Give the class a unique name:
    OperationClass.__name__= op_class_name
    return OperationClass


def Operation(fn, op_name, arity, univ_size):
    '''Return a UACalc Operation object defined using fn.
    
    :param fn: the Python function that will be used to define the UACalc operation.
    :param op_name: a string name of the resulting UACalc operation.
    :param arity: an integer arity of the operation.
    :param univ_size: an integer cardinality of the universe of the algebra.
    '''
    opClass = operation_class_factory(fn, op_name, arity)
    op = opClass(op_name, arity, univ_size)
    return op

